@@ -2,21 +2,12 @@
 
 import typing as t
 
-import aioredis
-from contextlib import asynccontextmanager
+from aioredis import BlockingConnectionPool, ConnectionPool, Redis, RedisError
 from asgi_tools._compat import json_dumps, json_loads
 
-from muffin import Application
 from muffin.plugins import BasePlugin
 
-try:
-    import fakeredis
-    from fakeredis import aioredis as fake_aioredis
-except ImportError:
-    fakeredis = None
-    fake_aioredis = None
-
-__version__ = "2.1.5"
+__version__ = "3.1.0"
 
 
 class Plugin(BasePlugin):
@@ -25,49 +16,49 @@ class Plugin(BasePlugin):
 
     name = 'redis'
     defaults = {
-        'address': 'redis://localhost',
+        'url': 'redis://localhost',
         'db': None,
         'password': None,
-        'poolsize': 10,
-        'fake': False,
         'encoding': 'utf-8',
+        'decode_responses': True,
         'jsonify': False,
+
+        'poolsize': 10,
+        'blocking': True,
+        'timeout': 20,
     }
 
     client = None
+    Error = RedisError
 
     def __getattr__(self, name):
         """Proxy attribute to self connection."""
         return getattr(self.client, name)
 
-    def setup(self, app: Application, **options):
-        """Check the configuration."""
-        super(Plugin, self).setup(app, **options)
-        if self.cfg.fake and not fake_aioredis:
-            raise RuntimeError('`fakeredis` is required to support `fake` mode.')
+    async def __aenter__(self):
+        await self.client.__aenter__()
+        return self
+
+    async def __aexit__(self, *args):
+        await self.client.__aexit__(*args)
 
     async def startup(self):
         """Setup a redis connection."""
-        params = {'db': self.cfg.db, 'password': self.cfg.password, 'encoding': self.cfg.encoding}
+        params = {
+            'db': self.cfg.db, 'password': self.cfg.password,
+            'decode_responses': self.cfg.decode_responses, 'encoding': self.cfg.encoding}
 
-        if self.cfg.fake:
-            if self.cfg.poolsize:
-                self.client = await fake_aioredis.create_redis_pool(
-                    fakeredis.FakeServer(), maxsize=self.cfg.poolsize, **params)
-            else:
-                self.client = await fake_aioredis.create_redis(fakeredis.FakeServer(), **params)
+        if self.cfg.blocking:
+            params['timeout'] = self.cfg.timeout
 
-        else:
-            if self.cfg.poolsize:
-                self.client = await aioredis.create_redis_pool(
-                    self.cfg.address, maxsize=self.cfg.poolsize, **params)
-            else:
-                self.client = await aioredis.create_redis(self.cfg.address, **params)
+        pool_cls = BlockingConnectionPool if self.cfg.blocking else ConnectionPool
+        pool = pool_cls.from_url(self.cfg.url, max_connections=self.cfg.poolsize, **params)
+        self.client = Redis(connection_pool=pool)
 
     async def shutdown(self):
         """Close the redis pool."""
-        self.client.connection.close()
-        await self.client.connection.wait_closed()
+        await self.client.close()
+        await self.client.connection_pool.disconnect()
 
     def set(self, key, value, *, jsonify: bool = None, **options) -> t.Awaitable:
         """Store the given value into Redis."""
@@ -98,18 +89,3 @@ class Plugin(BasePlugin):
                 pass
 
         return value
-
-    @asynccontextmanager
-    async def lock(self, key: str, ex: int = None):
-        """Simplest lock (before aioredis 2.0+)."""  # noqa
-
-        client = self.client
-        if client is None:
-            raise RuntimeError('Redis Plugin should be started')
-
-        lock_ = await client.set(key, '1', expire=ex, exist='SET_IF_NOT_EXIST')
-        try:
-            yield lock_
-        finally:
-            if lock_:
-                await client.delete(key)
